Type: Package
Package: openxlsx2
Title: Read, Write and Edit 'xlsx' Files
Version: 1.29
Language: en-US
Authors@R: c(
  person("Jordan Mark", "Barbone", email = "jmbarbone@gmail.com", role = "aut", comment = c(ORCID = "0000-0001-9788-3628")),
  person("Jan Marvin", "Garbuszus", email = "jan.garbuszus@ruhr-uni-bochum.de", role = c("aut", "cre")),
  person("Olivier", "Roy", role = "ctb"),
  person(family = "openxlsx authors", role = "cph", comment = "openxlsx package"),
  person("Arseny", "Kapoulkine", role = c("ctb", "cph"), comment = "Author of included pugixml code")
  )
Description: Simplifies the creation of 'xlsx' files by
    providing a high level interface to writing, styling and editing
    worksheets.
License: MIT + file LICENSE
URL: https://janmarvin.github.io/openxlsx2/,
        https://github.com/JanMarvin/openxlsx2
BugReports: https://github.com/JanMarvin/openxlsx2/issues
Depends: R (>= 3.5.0)
Imports: R6, Rcpp, grDevices, stringi, utils
LinkingTo: Rcpp
Suggests: encharter, ggplot2, knitr, mschart (>= 0.4), openssl,
        rmarkdown, rvg, testthat (>= 3.0.0), zip
VignetteBuilder: knitr
Encoding: UTF-8
Config/roxygen2/version: 8.1.0
Config/testthat/edition: 3
NeedsCompilation: yes
Packaged: 2026-08-20 06:40:07 UTC; janmarvingarbuszus
Author: Jordan Mark Barbone [aut] (ORCID:
    <https://orcid.org/0000-0001-9788-3628>),
  Jan Marvin Garbuszus [aut, cre],
  Olivier Roy [ctb],
  openxlsx authors [cph] (openxlsx package),
  Arseny Kapoulkine [ctb, cph] (Author of included pugixml code)
Maintainer: Jan Marvin Garbuszus <jan.garbuszus@ruhr-uni-bochum.de>
Repository: CRAN
Date/Publication: 2026-08-20 13:20:31 UTC
Built: R 4.5.2; aarch64-apple-darwin20; 2026-08-20 18:13:05 UTC; unix
Archs: openxlsx2.so.dSYM
