# Generated by roxygen2: do not edit by hand

S3method("+",fmt_txt)
S3method("[",wb_data)
S3method(as.character,fmt_txt)
S3method(as.character,pugi_xml)
S3method(print,fmt_txt)
S3method(print,pugi_xml)
S3method(xl_open,default)
S3method(xl_open,wbWorkbook)
export(apply_numfmt)
export(as_xml)
export(col2int)
export(convertToExcelDate)
export(convert_date)
export(convert_datetime)
export(convert_hms)
export(convert_to_excel_date)
export(create_border)
export(create_cell_style)
export(create_colors_xml)
export(create_colours_xml)
export(create_comment)
export(create_dxfs_style)
export(create_fill)
export(create_font)
export(create_hyperlink)
export(create_numfmt)
export(create_pivottablestyle)
export(create_shape)
export(create_sparklines)
export(create_tablestyle)
export(current_sheet)
export(dataframe_to_dims)
export(delete_data)
export(dims_to_dataframe)
export(dims_to_rowcol)
export(fmt_txt)
export(int2col)
export(na_strings)
export(next_sheet)
export(read_xlsx)
export(read_xml)
export(remove_comment)
export(rowcol_to_dims)
export(styles_on_sheet)
export(temp_xlsx)
export(validate_dims)
export(wbWorkbook)
export(wb_add_border)
export(wb_add_cell_style)
export(wb_add_chart_xml)
export(wb_add_chartsheet)
export(wb_add_comment)
export(wb_add_conditional_formatting)
export(wb_add_creators)
export(wb_add_data)
export(wb_add_data_table)
export(wb_add_data_validation)
export(wb_add_drawing)
export(wb_add_dxfs_style)
export(wb_add_encharter)
export(wb_add_fill)
export(wb_add_filter)
export(wb_add_font)
export(wb_add_form_control)
export(wb_add_formula)
export(wb_add_hyperlink)
export(wb_add_ignore_error)
export(wb_add_image)
export(wb_add_mips)
export(wb_add_mschart)
export(wb_add_named_region)
export(wb_add_named_style)
export(wb_add_numfmt)
export(wb_add_page_break)
export(wb_add_person)
export(wb_add_pivot_table)
export(wb_add_plot)
export(wb_add_slicer)
export(wb_add_sparklines)
export(wb_add_style)
export(wb_add_thread)
export(wb_add_timeline)
export(wb_add_worksheet)
export(wb_clean_sheet)
export(wb_clone_sheet_style)
export(wb_clone_worksheet)
export(wb_color)
export(wb_colour)
export(wb_comment)
export(wb_copy_cells)
export(wb_data)
export(wb_dims)
export(wb_freeze_pane)
export(wb_get_active_sheet)
export(wb_get_base_colors)
export(wb_get_base_colours)
export(wb_get_base_font)
export(wb_get_bookview)
export(wb_get_cell_style)
export(wb_get_comment)
export(wb_get_creators)
export(wb_get_mips)
export(wb_get_named_regions)
export(wb_get_order)
export(wb_get_person)
export(wb_get_properties)
export(wb_get_selected)
export(wb_get_sheet_names)
export(wb_get_sheet_visibility)
export(wb_get_tables)
export(wb_get_thread)
export(wb_grid_lines)
export(wb_group_cols)
export(wb_group_rows)
export(wb_load)
export(wb_merge_cells)
export(wb_open)
export(wb_page_setup)
export(wb_protect)
export(wb_protect_worksheet)
export(wb_read)
export(wb_remove_bookview)
export(wb_remove_col_widths)
export(wb_remove_comment)
export(wb_remove_conditional_formatting)
export(wb_remove_creators)
export(wb_remove_filter)
export(wb_remove_hyperlink)
export(wb_remove_named_region)
export(wb_remove_row_heights)
export(wb_remove_slicer)
export(wb_remove_tables)
export(wb_remove_timeline)
export(wb_remove_worksheet)
export(wb_save)
export(wb_set_active_sheet)
export(wb_set_base_colors)
export(wb_set_base_colours)
export(wb_set_base_font)
export(wb_set_bookview)
export(wb_set_cell_style)
export(wb_set_cell_style_across)
export(wb_set_col_widths)
export(wb_set_creators)
export(wb_set_grid_lines)
export(wb_set_header_footer)
export(wb_set_last_modified_by)
export(wb_set_order)
export(wb_set_page_setup)
export(wb_set_properties)
export(wb_set_row_heights)
export(wb_set_selected)
export(wb_set_sheet_names)
export(wb_set_sheet_visibility)
export(wb_set_sheetview)
export(wb_to_df)
export(wb_ungroup_cols)
export(wb_ungroup_rows)
export(wb_unmerge_cells)
export(wb_update_table)
export(wb_workbook)
export(write_comment)
export(write_data)
export(write_datatable)
export(write_formula)
export(write_xlsx)
export(xl_open)
export(xml_add_child)
export(xml_attr)
export(xml_attr_mod)
export(xml_node)
export(xml_node_create)
export(xml_node_name)
export(xml_rm_child)
export(xml_value)
import(R6)
import(Rcpp)
importFrom(grDevices,bmp)
importFrom(grDevices,col2rgb)
importFrom(grDevices,colors)
importFrom(grDevices,dev.copy)
importFrom(grDevices,dev.list)
importFrom(grDevices,dev.off)
importFrom(grDevices,jpeg)
importFrom(grDevices,palette)
importFrom(grDevices,png)
importFrom(grDevices,rgb)
importFrom(grDevices,tiff)
importFrom(stringi,stri_encode)
importFrom(stringi,stri_extract_all_regex)
importFrom(stringi,stri_extract_first_regex)
importFrom(stringi,stri_join)
importFrom(stringi,stri_match_first_regex)
importFrom(stringi,stri_opts_collator)
importFrom(stringi,stri_order)
importFrom(stringi,stri_pad_left)
importFrom(stringi,stri_rand_strings)
importFrom(stringi,stri_read_lines)
importFrom(stringi,stri_replace_all_fixed)
importFrom(stringi,stri_replace_all_regex)
importFrom(stringi,stri_split_fixed)
importFrom(stringi,stri_sub)
importFrom(stringi,stri_unescape_unicode)
importFrom(utils,download.file)
importFrom(utils,head)
importFrom(utils,menu)
importFrom(utils,read.csv)
importFrom(utils,unzip)
importFrom(utils,zip)
useDynLib(openxlsx2, .registration=TRUE)
