% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils.R
\name{con_rng}
\alias{con_rng}
\title{consecutive range in vector}
\usage{
con_rng(x)
}
\arguments{
\item{x}{integer vector}
}
\description{
consecutive range in vector
}
\keyword{internal}
