% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/dates.R
\name{convert_to_excel_date}
\alias{convert_to_excel_date}
\title{convert back to an Excel Date}
\usage{
convert_to_excel_date(df, date1904 = FALSE)
}
\arguments{
\item{df}{dataframe}

\item{date1904}{take different origin}
}
\description{
convert back to an Excel Date
}
\examples{
 xlsxFile <- system.file("extdata", "openxlsx2_example.xlsx", package = "openxlsx2")
 wb1 <- wb_load(xlsxFile)
 df <- wb_to_df(wb1)
 # conversion is done on dataframes only
 convert_to_excel_date(df = df["Var5"], date1904 = FALSE)
}
