% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/wb_styles.R
\name{styles_on_sheet}
\alias{styles_on_sheet}
\title{Get all styles on a sheet}
\usage{
styles_on_sheet(wb, sheet)
}
\arguments{
\item{wb}{workbook}

\item{sheet}{worksheet}
}
\description{
Get all styles on a sheet
}
